# Creating Your First elm-vegaLite visualization

## 1. One-time setup of Elm

If you already have [Elm](http://elm-lang.org) installed, you can skip this section. If not, you can install Elm by following [these instructions](https://guide.elm-lang.org/install.html). While not absolutely necessary, I would strongly recommend configuring your favourite text editor to be able to format, syntax highlight and lint Elm code (details in the installation guide above).

My preferred editor that accepts Elm-specific plug-ins is [VSCode](https://code.visualstudio.com), but any text editor will do.

## 2. Create HTML/JavaScript page to link with Elm

The easiest way to get elm-vegaLite to produce graphical output is to write a skeleton Elm program that sends some Vega-Lite to an HTML page. That HTML page will contain some JavaScript that turns the Vega-Lite into graphics.

To get started, copy this example to a file `helloWorld.html` somewhere on your machine:

```html
<!DOCTYPE html>

<head>
  <title>Hello World from elm-vegaLite</title>
  <meta charset="utf-8">

  <!-- These scripts link to the Vega/Vega-Lite runtime -->
  <script src="https://cdn.jsdelivr.net/npm/vega@5"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega-lite@5"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega-embed@6"></script>

  <!-- This is the script generated from Elm -->
  <script src="js/helloWorld.js"></script>
</head>

<body>
  <!-- The id indicates the container in which to embed the Vega-Lite output. -->
  <div id="vis"></div>

  <script>
    // This function grabs the specification from Elm (here called specs).
    // Replace 'HelloWorld' in line below with name of your Elm module when you write your own.
    Elm.HelloWorld.init().ports.elmToJS.subscribe(function(specs) {
      // Change actions to true to display links to source, editor and image.
      vegaEmbed("#vis", specs, {actions: false}).catch(console.warn);
    });
  </script>

</body>

</html>
```

Most of this is scaffolding that you won't need to change as you write new elm-vegaLite applications. The key elements are:

- Links to `vega`, `vega-lite` and `vega-embed` JavaScript that do the hard work of converting the Vega-Lite spec you will be generating into graphical output.

- A link to the JavaScript file generated by elm-vegaLite that creates your Vega-Lite specification (here called `helloWorld.js`)

- A `<div>` container with an ID (here called `vis`) that will be the location in the HTML page that will contain your visualization.

- An _Elm subscription_ that links this page with the Elm program that generates the Vega-Lite specification Here `Elm.HelloWorld.init()` references an Elm module that must be called `HelloWorld`. When you create elm-vegaLite modules with different names, you must match that name here (for example, `Elm.MyNewModule.init()`).

## 3. Create an elm-vegaLite Program

Create a file called `HelloWorld.elm` in the same location as `helloWorld.html`.
Copy the following code into it.

```elm
port module HelloWorld exposing (elmToJS)

import Platform
import VegaLite exposing (..)


myVis : Spec
myVis =
    let
        data =
            dataFromColumns []
                << dataColumn "x" (nums [ 10, 20, 30 ])

        enc =
            encoding
                << position X [ pName "x", pQuant ]
    in
    toVegaLite
        [ title "Hello, World!" []
        , data []
        , enc []
        , circle []
        ]



{- The code below is boilerplate for creating a headless Elm module that opens
   an outgoing port to JavaScript and sends the Vega-Lite spec (myVis) to it.
-}


main : Program () Spec msg
main =
    Platform.worker
        { init = always ( myVis, elmToJS myVis )
        , update = \_ model -> ( model, Cmd.none )
        , subscriptions = always Sub.none
        }


port elmToJS : Spec -> Cmd msg
```

This creates a working Elm-program whose job is to send a Vega-Lite specification as a JSON file via an Elm _port_ to the JavaScript seen in the HTML file above. For convenience, we will always call this port `elmToJS` so it need not be changed when creating new programs. The only code you need to modify as you write your own elm-vegaLite is the function that generates a `Spec`, here called `myVis` and the name of the module (here called `HelloWorld`). Note also that values in the `init` tuple in `main` must correspond to the name of the function generating the spec (here called `myVis`).

`myVis` in this example generates a single data column called `x` containing just three numbers (10, 20 and 30). It symbolises each with a `circle` mark and positions each along a horizontal (`X`) axis below a welcoming title.

## 4. Compile the elm-vegaLite into JavaScript

The final task is to convert the Elm file into JavaScript so it can be used by the HTML. To do this, open a command line window (e.g. a Terminal on MacOS or PowerShell on Windows), change to the directory containing your HTML and Elm files and type

```sh
elm make helloWorld.elm --output=js/helloWorld.js --optimize
```

This should create the `helloWorld.js` file required by the HTML. If you are making use of elm `Debug` functions while you develop your code, you will need to drop the `--optimize` flag.

Opening `helloWorld.html` in the browser should display the graphical output similar to this:

![Hello, World! output](images/helloWorld.png)

Congratulations on creating your first elm-vegaLite visualization!

_Note that in this example the data used in the visualization is generated entirely inline and does not rely on external files. Typical elm-vegaLite programs will however load data from external .json, .csv or other files and so normal JavaScript file loading restrictions apply. As such you would more typically test your applications with a local web server (such as generated by `python -m http.server` or `elm-reactor`) rather than load directly from your file system._

## 5. Next Steps

Most of the setup described above varies little between applications. To understand more about how elm-vegaLite itself works, and how to encode different visualization specifications, have a look at

- embedding [multiple Visualizations in a single page](../helloWorlds/README.md),
- the [elm-vegaLite Walkthrough](../walkthrough/README.md),
- some [examples](../../examples/README.md),
- the [gallery](../../gallery/README.md) for a full range of examples.
